unit NDSP;
{$MINENUMSIZE 4}
interface

uses Windows;

const
  NSP_EPS  = 1.0E-12;
  NSP_PI   = 3.14159265358979324; (* Standard C not support M_PI     *)
  NSP_2PI  = 6.28318530717958648;
  NSP_PI_2 = 1.57079632679489662;
  NSP_PI_4 = 0.785398163397448310;

type
{$IFDEF VER90}
  PDouble = ^Double;
{$ENDIF}
  Short = Smallint;
  Long	= Longint;
  Float = Single;

  PSCplx = ^TSCplx;
  TSCplx = record
    re: float;
    im:	float;
  end;

  PDCplx = ^TDCplx;
  TDCplx = record
    re: Double;
    im: Double;
  end;

  PWCplx = ^TWCplx;
  TWCplx = record
    re: Short;
    im: Short;
  end;

  PICplx = ^TICplx;
  TICplx = record
    re: Integer;
    im: Integer;
  end;

  ArrayIndex = 0..99999999;	// 0..10**8-1

  TByteArray = array[ArrayIndex] of Byte;	PByteArray = ^TByteArray;
  TShortArray = array[ArrayIndex] of Short;	PShortArray = ^TShortArray;
  TIntArray = array[ArrayIndex] of Integer;	PIntArray = ^TIntArray;
  TLongArray = array[ArrayIndex] of Long;	PLongArray = ^TLongArray;
  TFloatArray = array[ArrayIndex] of Float;	PFloatArray = ^TFloatArray;
  TDoubleArray = array[ArrayIndex] of Double;	PDoubleArray = ^TDoubleArray;
  TWCplxArray = array[ArrayIndex] of TWCplx;	PWCplxArray = ^TWCplxArray;
  TSCplxArray = array[ArrayIndex] of TSCplx;	PSCplxArray = ^TSCplxArray;
  TDCplxArray = array[ArrayIndex] of TDCplx;	PDCplxArray = ^TDCplxArray;
  TICplxArray = array[ArrayIndex] of TICplx;	PICplxArray = ^TICplxArray;

  TNspFourierTrOptions = set of (
        NSP_Forw,
        NSP_Inv,
        NSP_Init,
        NSP_Free,
        NSP_NoScale,
        NSP_NoBitRev,
        NSP_InBitRev,
        NSP_OutBitRev,
        NSP_OutRCPack,
        NSP_OutRCPerm,
        NSP_InRCPack,
        NSP_Bit11,               // reserved
        NSP_InRCPerm,
        NSP_DoIntCore,
        NSP_DoFloatCore,
        NSP_DoFastMMX);


  TNSPSRandUniState = record
    Low, Muler: Float;		// Lower value and scale multiplier */
    Seed2, Carry: UInt;		// Variables of the algorithm       */
    seed1: array[0..2] of UInt;	// Variables of the algorithm       */
    QuickSeed: Integer;
  end;



  TNSPSRandGausState = record
    Mean, StDev: Float;		// Mean and standard deviation values */
    ExtraValue: Float;		// Value computed early               */
    UseExtra: Integer;		// Flag to use extra value            */
    Seed2, Carry: UInt;		// Variables of the algorithm         */
    Seed1: array[0..2] of UInt;	// Variables of the algorithm         */
    QuickSeed: UInt;
  end;

//--- Tone structures -----------------------------------------------------*/

const
  NSP_DCT_Forward      = 1;
  NSP_DCT_Inverse      = 2;
  NSP_DCT_Free         = 8;


type
  TFirLmsTapState = record
    Taps: Pointer;
    Len: Integer;
    UpFactor: Integer;
    UpPhase: Integer;
    DownFactor: Integer;
    DownPhase: Integer;
    IsMultiRate: Integer;
    IsFilterMode: Integer;
  end;

  TFirLmsDlyState = record
    Dlyl: Pointer;
    Len: Integer;
    IsFilterMode: Integer;
  end;

//* --- Finite impulse response filter -------------------------------------*/

  TNSPFirTapState = record
    Taps: Pointer;
    Len: Integer;
    UpFactor: Integer;
    UpPhase: Integer;
    DownFactor: Integer;
    DownPhase: Integer;
    IsMultiRate: Integer;
    IsFilterMode: Integer;
    TapsFactor: Integer;
  end;

//* --- Delay line of FIR low level filter ---------------------------------*/

  TNSPFirDlyState = TFirLmsDlyState;


type
  TFirSect = record
    num: Integer;                             // section number                  */
    len: Integer;                             // section taps length in use      */
    xId: Integer;                             // section input data index        */
    tId: Integer;                             // section taps index              */
  end; // firSect
  TFirSections = array[ArrayIndex] of TFirSect;

type
  TNSPFirState = record
    upFactor:	Integer;		// up                              */
    upPhase:	Integer;                //  parameters                     */
    downFactor:	Integer;                // down                            */
    downPhase:	Integer;                //  parameters                     */
    isMultiRate:Integer;                // multi-rate mode flag            */
    isInit:	Pointer;                // init flag                       */

    TapsLen:	Integer;                // taps length in use              */
    tapsBlk:	Integer;
    taps:	Pointer;                // taps pointer in use             */

    dlylLen:	Integer;                // delay line length               */
    inpLen:	Integer;                // input buffer length             */
    useInp:	Integer;                // input buffer length used        */
    dlyl:	Pointer;		// extended filter delay line      */
    dlylLimit:	Pointer;		// delay line buffer end pointer   */
    useDlyl:	Pointer;		// delay line pointer in use       */

    sect: ^TFirSections;		// FIR sections                    */

    tapsFactor:	Integer;		// taps scale factor                */
    utapsLen:	Integer;		// number of filter taps (user def) */
  end;	// TNSPFirState


procedure nspsFirInit(TapVals: PFloatArray; TapsLen: Integer;
			  DlyVals: PFloatArray; var State: TNSPFirState); stdcall;

procedure nspsFirInitMr(TapVals: PFloatArray; TapsLen: Integer;
			    DlyVals: PFloatArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); stdcall;
procedure nspFirFree(var State: TNSPFirState); stdcall;

function nspsFir(var State: TNSPFirState; samp: Float): Float; stdcall;

procedure nspsbFir(var State: TNSPFirState; inSamps: PFloatArray;
		       outSamps: PFloatArray; numIters: Integer); stdcall;


procedure nspsRandUniInit(Seed, Low, High: Float; var State: TNSPSRandUniState); stdcall;

procedure nspsbRandUni(var State: TNSPSRandUniState; Samps: PFloatArray; SampsLen: Integer); stdcall;

procedure nspsRandGausInit(Seed, Mean, StDev: Float; var State: TNSPSRandGausState); stdcall;

procedure nspsbRandGaus(var State: TNSPSRandGausState; Samps: PFloatArray; SampsLen: Integer); stdcall;

implementation

const nspl = 'DSP.DLL';

  procedure nspsFirInit(TapVals: PFloatArray; TapsLen: Integer;
			  DlyVals: PFloatArray; var State: TNSPFirState); external nspl;

  procedure nspsFirInitMr(TapVals: PFloatArray; TapsLen: Integer;
			    DlyVals: PFloatArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); external nspl;
  procedure nspFirFree(var State: TNSPFirState); external nspl;

  function nspsFir(var State: TNSPFirState; samp: Float): Float; external nspl;

  procedure nspsbFir(var State: TNSPFirState; inSamps: PFloatArray;
		       outSamps: PFloatArray; numIters: Integer); external nspl;
  procedure nspsRealFftl(
	Samps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspwRealFftlNip(
	inSamps: PShortArray;
	outSamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;


  procedure nspsRandUniInit(Seed, Low, High: Float; var State: TNSPSRandUniState); external nspl;
  procedure nspsbRandUni(var State: TNSPSRandUniState; Samps: PFloatArray; SampsLen: Integer); external nspl;
  procedure nspsRandGausInit(Seed, Mean, StDev: Float; var State: TNSPSRandGausState); external nspl;
  procedure nspsbRandGaus(var State: TNSPSRandGausState; Samps: PFloatArray; SampsLen: Integer); external nspl;


end.

